@@ -13,49 +13,6 @@
 # You should have received a copy of the GNU General Public License
 # along with dynsbm.  If not, see <http://www.gnu.org/licenses/>
 
-
-alluvial.plot <- function(dynsbm, timestep.abbrev="T", minimal.flow=1, only.present=FALSE){
-                                        #library(riverplot)
-    if (! "dynsbm" %in% class(dynsbm)) stop("Object must be of class 'dynsbm'")
-    T <- ncol(dynsbm$membership)
-    membership <- dynsbm$membership
-    if(only.present) membership[membership==0] <- NA
-    edges <- c()
-    for (t in 1:(T-1)){
-        groups1 <- membership[,t]
-        groups2 <- membership[,t+1]
-        tb <- table(groups1,groups2)
-        groupname1 <- dimnames(tb)$groups1
-        groupname2 <- dimnames(tb)$groups2
-        for (q in 1:dim(tb)[1])
-            for (l in 1:dim(tb)[2]){
-                if (tb[q,l]>=minimal.flow){
-                    if(groupname1[q]=="0" && groupname2[l]=="0"){ # SKIP 0->0
-                    } else{
-                        edges <- rbind(edges, c(paste(timestep.abbrev,t,"-",groupname1[q],sep=""), paste(timestep.abbrev,t+1,"-",groupname2[l],sep=""), tb[q,l]))
-                    }
-                }
-            }
-    }
-    edges <- data.frame(N1 = as.character(edges[,1]),
-			N2 = as.character(edges[,2]),
-			Value=as.numeric(edges[,3]),
-			stringsAsFactors = F)
-    nodes <- data.frame(ID = unique(c(edges$N1, edges$N2)), stringsAsFactors = F)
-    vID <- strsplit(nodes$ID,"-")
-    nodes$x <- as.numeric(substr(sapply(vID, FUN=function(v){return(v[1])}),2,5))
-    nodes$y <- as.numeric(sapply(vID, FUN=function(v){return(v[2])}))
-    rownames(nodes) <- nodes$ID
-    
-    styles <- lapply(nodes$y, function(yy) {
-        list(col = ifelse(yy==0,colors()[1],colors()[200]), lty = 0, textcol = "black")
-    })
-    names(styles) <- nodes$ID
-    rp <- list(nodes = nodes, edges = edges, styles = styles)
-    class(rp) <- c(class(rp), "riverplot")
-    plot(rp, nodewidth = .5, plot_area = 0.95)
-}
-
 gradient.polygon <- function(x1,x2,y1,y2,col1,col2,ylim.low){
     gap <- (x2-x1)/20
     a <- (y2-y1)/(x2-x1)
@@ -80,7 +37,8 @@ connectivity.plot <- function(dynsbm, Y){
     T <- ncol(dynsbm$membership)
     Q <- nrow(dynsbm$trans)
     membership <- dynsbm$membership
-    old.mar <- par()$mar
+    oldpar <- par(no.readonly = TRUE)
+    on.exit(par(oldpar))
     par(mar = rep(0.33, 4))
     par(mfrow=c(Q,Q))
     ## BETA
@@ -194,8 +152,8 @@ connectivity.plot <- function(dynsbm, Y){
             }
         }
     }
-    par(mar=old.mar)
-    par(mfrow=c(1,1))
+    par(oldpar)
+    NULL
 }
 
 adjacency.plot <- function(dynsbm, Y, present=NULL, col=heat.colors(9)){
@@ -208,6 +166,8 @@ adjacency.plot <- function(dynsbm, Y, present=NULL, col=heat.colors(9)){
     if(!("gamma" %in% names(dynsbm)) && !("mu" %in% names(dynsbm))){
         col=col[1]
     }
+    oldpar <- par(no.readonly = TRUE)
+    on.exit(par(oldpar))
     par(mfrow=c(min(dim(Y)[1],3), ceiling(dim(Y)[1]/(min(dim(Y)[1],3)))))
     par(mar = rep(0.5,4))
     for(t in 1:dim(Y)[1]){
@@ -227,4 +187,6 @@ adjacency.plot <- function(dynsbm, Y, present=NULL, col=heat.colors(9)){
             image(matrix(1))
         }
     }
+    par(oldpar)
+    NULL
 }
